import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class CartesianPlot extends JFrame {
    public ArrayList<Polygon> polygons;
    public ArrayList<Point> points;
    public ArrayList<Color> poly_colors;
    public ArrayList<Color> point_colors;

    public int PAD = 50;
    public int MAX_X = 150;
    public int MAX_Y = 150;

    // This constructor receives as an input
    // the maximum x and y cooridnates
    public CartesianPlot(int max_x, int max_y) {
        super("CartesianPlot");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        MAX_X = max_x;
        MAX_Y = max_y;
        Plot map = new Plot();
        add(map);
        setSize(600, 600);
        setVisible(true);
        polygons = new ArrayList<Polygon>();
        points = new ArrayList<Point>();
        point_colors = new ArrayList<Color>();
        poly_colors = new ArrayList<Color>();
    }

    public void addPoint(Point p) {
        points.add(p);
        int j = polygons.size() + points.size();
        Color c = new Color((j * 101) % 200, (j * 73) % 200, (j * 67) % 200,
                220);
        point_colors.add(c);
    }

    public void addPolygon(Polygon p) {
        polygons.add(p);
        int j = polygons.size() + points.size();
        Color c = new Color((j * 101) % 200, (j * 73) % 200, (j * 67) % 200,
                220);
        poly_colors.add(c);
    }

    public class Plot extends JPanel {
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2 = (Graphics2D) g;
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);

            g2.setColor(Color.black);
            g.setFont(new Font("default", Font.PLAIN, 14));

            int w = getWidth();
            int h = getHeight();
            int pointSize = Math.max(Math.min(w, h) / 60, 10);

            double scale_x = (double) (w - 2 * PAD) / MAX_X;
            double scale_y = (double) (h - 2 * PAD) / MAX_Y;

            double xInc = (double) (w - 2 * PAD) / (MAX_X - 1);
            double scale = (double) (h - 2 * PAD) / MAX_Y;
            // Draw abcissa.
            int tickInc = MAX_X / 10;
            for (int i = 0; i <= MAX_X; i += tickInc) {
                int x = PAD + (int) (i * xInc);
                int y = h - PAD;
                g.drawString(Integer.toString(i), x - 5, y + 20);
                g2.draw(new Line2D.Double(x, y - 5, x, y + 5));
            }
            g2.draw(new Line2D.Double(PAD, h - PAD, w - PAD / 2, h - PAD));
            AffineTransform orig = g2.getTransform();
            g2.rotate(-Math.PI / 2);
            g2.setColor(Color.black);
            g2.drawString("Y", -((h + PAD) / 2), PAD / 3);
            g2.setTransform(orig);

            // Draw ordinate.
            tickInc = MAX_Y / 10;

            for (int i = tickInc; i < h - PAD; i += tickInc) {
                int x = PAD;
                int y = h - PAD - (int) (i * scale);
                String tickMark = Integer.toString(i);
                if (MAX_Y > 30){
                    int closest_10 = (int) (Math.round((i / scale) / 10) * 10);
                    y = h - PAD - (int) (closest_10 * scale);
                    tickMark = Integer.toString(closest_10);
                }
                if (y < PAD)
                    break;
                
                int stringLen = (int) g2.getFontMetrics()
                        .getStringBounds(tickMark, g2).getWidth();
                g.drawString(tickMark, x - stringLen - 8, y + 5);
                g2.draw(new Line2D.Double(x - 5, y, x + 5, y));
            }
            g2.draw(new Line2D.Double(PAD, PAD / 2, PAD, h - PAD));
            g.drawString("X", (w - PAD) / 2, h - PAD + 40);

            for (int i = 0; i < polygons.size(); i++) {
                Polygon poly = polygons.get(i);
                g2.setColor(poly_colors.get(i));
                System.out.println(poly);
                // draw polygon
                for (int j = 0; j < poly.vertices.length; j++) {
                    Point p = poly.vertices[j];
                    Point q = poly.vertices[(j + 1) % poly.vertices.length];

                    int pX = PAD + (int) ((p.x) * scale_x);
                    int pY = h - PAD - (int) (p.y * scale_y);
                    int qX = PAD + (int) ((q.x) * scale_x);
                    int qY = h - PAD - (int) (q.y * scale_y);

                    g2.fill(new Ellipse2D.Double(pX - pointSize / 2, pY
                            - pointSize / 2, pointSize, pointSize));

                    g2.drawLine(pX, pY, qX, qY);
                }
            }

            for (int i = 0; i < points.size(); i++) {
                Point p = points.get(i);
                g2.setColor(point_colors.get(i));

                int pX = PAD + (int) ((p.x) * scale_x);
                int pY = h - PAD - (int) (p.y * scale_y);

                g2.fill(new Ellipse2D.Double(pX - pointSize / 2, pY - pointSize
                        / 2, pointSize, pointSize));

            }
        }
    }
}
